@@ -36,7 +36,7 @@
 #define TMP_LEN 16
 #define LARGE_LEN 65536
 
-static char rcsid[] = "$Id: pnpdump.c,v 1.15 1998/05/25 17:22:16 fox Exp $";
+static char rcsid[] = "$Id: pnpdump.c,v 1.16 1998/10/09 22:19:06 fox Exp $";
 
 #ifdef REALTIME
 /*
@@ -203,7 +203,6 @@ char devid[8];											   /* Copy as devidstr
 														    * returns pointer to
 														    * static buffer subject
 														    * to change */
-int logdevno;
 int nestdepth;
 int curid;
 int curdma, depdma;
@@ -213,10 +212,10 @@ int curmem, depmem;
 int starteddeps;
 char *indep = "";
 
+/* This shouldn't be required 8-Oct-1998 */
 static void
 prepare_to_print(void)
 {
-	logdevno = 0;
 	nestdepth = 0;
 	curid = 0;
 	curdma = 0;
@@ -234,6 +233,7 @@ prepare_to_print(void)
 unsigned char serial_identifier[NUM_CARDS + 1][IDENT_LEN];
 unsigned char tmp[TMP_LEN];
 unsigned char large[LARGE_LEN];
+static char *devstring = 0;
 
 int showmasks = 0;
 int do_autoconfig = 0;
@@ -251,9 +251,9 @@ void dumpdata(int);
 void read_idents(void);
 
 static void usage(char *program_name);
-#ifdef DUMPREGS
+
+int do_dumpregs = 0;
 void dumpregs(int);
-#endif /* DUMPREGS */
 
 char *devidstr(unsigned char, unsigned char, unsigned char, unsigned char);
 
@@ -276,11 +276,12 @@ main(int argc, char **argv)
 		{"config", no_argument, NULL, 'c'},
 		{"script", optional_argument, NULL, 's'},
 		{"reset", no_argument, NULL, 'r'},
+		{"dumpregs", no_argument, NULL, 'd'},
 		{0, 0, 0, 0},
 	};
 	int opt;
 
-	while ((opt = getopt_long(argc, argv, "cms:", longopts, NULL)) != EOF)
+	while ((opt = getopt_long(argc, argv, "cms:rd", longopts, NULL)) != EOF)
 	{
 		switch (opt)
 		{
@@ -297,6 +298,9 @@ main(int argc, char **argv)
 			do_script = 1;
 			output_script = optarg;
 			break;
+		case 'd':
+			do_dumpregs = 1;
+			break;
 		case '?':
 #if 0
 			fprintf(stderr, "unrecognized option.\n");
@@ -330,6 +334,10 @@ main(int argc, char **argv)
 		{
 			showmasks = 1;
 		}
+		else if (strcmp(argv[1], "--dumpregs") == 0)
+		{
+			do_dumpregs = 1;
+		}
 		else if (strcmp(argv[1], "--config") == 0)
 		{
 			do_autoconfig = 1;
@@ -408,9 +416,6 @@ main(int argc, char **argv)
 #ifdef NEEDNANOSLEEP
 	printf(" -DNEEDNANOSLEEP");
 #endif /* NEEDNANOSLEEP */
-#ifdef DUMPREGS
-	printf(" -DDUMPREGS");
-#endif /* DUMPREGS */
 	printf("\n#\n");
 	initiate();
 	if (do_autoconfig)
@@ -426,9 +431,8 @@ main(int argc, char **argv)
 #else
 		dumpdata(i);
 #endif
-#ifdef DUMPREGS
-		dumpregs(i);
-#endif /* DUMPREGS */
+		if(do_dumpregs)
+			dumpregs(i);
 	}
 	if (do_autoconfig)
 	{
@@ -500,9 +504,9 @@ usage(char *program_name)
 			"http://www.roestock.demon.co.uk/isapnptools/\n"
 			"This software has NO WARRANTY, use at your OWN RISK\n\n"
 #ifndef __DJGPP__
-			"Usage: %s [--masks] [--config] [--reset] [--script=outputfile]\n",
+			"Usage: %s [--masks] [--config] [--reset] [--script=outputfile] [--dumpregs]\n",
 #else
-			"Usage: %s [--masks] [--config] [--reset]\n",
+			"Usage: %s [--masks] [--config] [--reset] [--dumpregs]\n",
 #endif
 			rcsid, program_name);
 }
@@ -614,9 +618,9 @@ initiate(void)
 			printf("\n");
 		}
 		if(do_fullreset)
-			printf("\n# (DEBUG)\n(READPORT 0x%04x)\n(ISOLATE CLEAR)\n(IDENTIFY *)\n\n", port);
+			printf("\n# (DEBUG)\n(READPORT 0x%04x)\n(ISOLATE CLEAR)\n(IDENTIFY *)\n", port);
 		else
-			printf("\n# (DEBUG)\n(READPORT 0x%04x)\n(ISOLATE PRESERVE)\n(IDENTIFY *)\n\n", port);
+			printf("\n# (DEBUG)\n(READPORT 0x%04x)\n(ISOLATE PRESERVE)\n(IDENTIFY *)\n", port);
 	}
 	else
 	{
@@ -624,8 +628,9 @@ initiate(void)
 		WRITE_DATA(CONFIG_WAIT_FOR_KEY);
 		sendkey();
 		read_idents();
-		printf("\n# (DEBUG)\n(READPORT 0x%04x)\n(CSN %d)\n(IDENTIFY *)\n\n", read_port, boards_found);
+		printf("\n# (DEBUG)\n(READPORT 0x%04x)\n(CSN %d)\n(IDENTIFY *)\n", read_port, boards_found);
 	}
+	printf("(VERBOSITY 2)\n(CONFLICT (IO FATAL)(IRQ FATAL)(DMA FATAL)(MEM FATAL)) # or WARNING\n\n");
 }
 
 void
@@ -1042,6 +1047,7 @@ read_board_resources(struct resource *first_element,
 static void
 print_resource(FILE * output_file, struct resource *res, int selected)
 {
+	static int logdevno = 0;
 	int type = res->type;
 	int len = res->len;
 	int i;
@@ -1053,6 +1059,7 @@ print_resource(FILE * output_file, struct resource *res, int selected)
 		{
 			int csn = res->start;
 
+			logdevno = 0;
 			serialno = (unsigned long)
 				serial_identifier[csn][4] +
 				(serial_identifier[csn][5] << 8) +
@@ -1326,6 +1333,14 @@ print_resource(FILE * output_file, struct resource *res, int selected)
 								  selected && (res->value == i));
 			}
 			indep = "    ";
+			/* If we have a device name, show it (the last description string before the END DF flag is it) */
+			if (serialno == 0xffffffffUL)
+				printf(" (NAME \"%s/-1[%d]", devid, logdevno-1);
+			else
+				printf(" (NAME \"%s/%lu[%d]", devid, serialno, logdevno-1);
+			if(devstring)
+				printf("{%-20s}", devstring);
+			printf("\")\n");
 			printf("\n# %sEnd dependent functions\n", indep);
 			break;
 		}
@@ -1348,8 +1363,8 @@ print_resource(FILE * output_file, struct resource *res, int selected)
 				printf("# %s%s0x%04x..0x%04x\n", indep, indep, indep, i, i + res->data[6] - 1);
 			}
 #endif /* DUMPADDR */
-			printf("%s (IO %d (BASE 0x%04lx))\n",
-				   comment_if_not_selected, curio, res->value);
+			printf("%s (IO %d (SIZE %d) (BASE 0x%04lx))\n",
+				   comment_if_not_selected, curio, res->data[6], res->value);
 			curio++;
 			if (!starteddeps)
 				depio = curio;
@@ -1365,8 +1380,8 @@ print_resource(FILE * output_file, struct resource *res, int selected)
 			}
 			printf("# %sFixed IO base address 0x%04lx\n", indep, res->start);
 			printf("# %s%sNumber of IO addresses required: %ld\n", indep, indep, res->size);
-			printf("%s (IO %d (BASE 0x%04lx))\n",
-				   comment_if_not_selected, curio, res->start);
+			printf("%s (IO %d (SIZE %ld) (BASE 0x%04lx))\n",
+				   comment_if_not_selected, curio, res->size, res->start);
 			curio++;
 			if (!starteddeps)
 				depio = curio;
@@ -1427,16 +1442,36 @@ print_resource(FILE * output_file, struct resource *res, int selected)
 	case ANSIstr_TAG:
 		{
 			printf("# %sANSI string -->", indep);
+			/* Remember the device name */
+			if(devstring)
+				free(devstring);
+			devstring = (char *)malloc(len + 1);
+			if(!devstring)
+			{
+				fprintf(stderr, "Out of memory to store board device string\n");
+				exit(1);
+			}
 			for (i = 0; i < len; i++)
-				putchar(res->data[i]);
+				putchar(devstring[i] = res->data[i]);
+			devstring[i] = 0;
 			printf("<--\n");
 			break;
 		}
 	case UNICODEstr_TAG:
 		{
 			printf("# %sUNICODE string -->", indep);
+			/* Remember the device name */
+			if(devstring)
+				free(devstring);
+			devstring = (char *)malloc(len + 1);
+			if(!devstring)
+			{
+				fprintf(stderr, "Out of memory to store board device string\n");
+				exit(1);
+			}
 			for (i = 0; i < len; i++)
-				putchar(res->data[i]);
+				putchar(devstring[i] = res->data[i]);
+			devstring[i] = 0;
 			printf("<--\n");
 			break;
 		}
@@ -1463,6 +1498,10 @@ print_resource(FILE * output_file, struct resource *res, int selected)
 				}
 				printf("\n");
 			}
+			/* Cancel any device names */
+			if(devstring)
+				free(devstring);
+			devstring = 0;
 			printf("# %sEnd tag... Checksum 0x%02x (%s)\n\n", indep, csum, (csum % 256) ? "BAD" : "OK");
 			break;
 		}
@@ -1873,7 +1912,6 @@ dumpdata(int csn)
 }
 #endif
 
-#ifdef DUMPREGS
 void
 dumpregs(int csn)
 {
@@ -1881,8 +1919,27 @@ dumpregs(int csn)
 	int i;
 	int addr;
 	int desc;
+	unsigned long serialno = (unsigned long) serial_identifier[csn][4] + (serial_identifier[csn][5] << 8) +
+	(serial_identifier[csn][6] << 16) + (serial_identifier[csn][7] << 24);
+	strncpy(devid, devidstr(serial_identifier[csn][0],
+							serial_identifier[csn][1],
+							serial_identifier[csn][2],
+							serial_identifier[csn][3]), 8);
+
 	Wake(csn);
-	printf("\nConfiguration regsisters\n\nVendor defined card level registers 0x20..2f:");
+	printf("# Configuration registers for card %d: (serial identifier", csn);
+	for (i = IDENT_LEN; i--;)
+		printf(" %02x", serial_identifier[csn][i]);
+	printf(")\n");
+	if (serialno == 0xffffffffUL)
+	{
+		printf("# Vendor Id %s, No Serial Number (-1), checksum 0x%02X.\n", devid, serial_identifier[csn][8]);
+	}
+	else
+	{
+		printf("# Vendor Id %s, Serial Number %lu, checksum 0x%02X.\n", devid, serialno, serial_identifier[csn][8]);
+	}
+	printf("#\n# Vendor defined card level registers 0x20..2f:");
 	for (addr = 0x20; addr < 0x30; addr++)
 	{
 		ADDRESS(addr);
@@ -1895,10 +1952,10 @@ dumpregs(int csn)
 		WRITE_DATA(logical_device);
 		if (READ_DATA != logical_device)
 			break;
-		printf("Logical device %d\n", logical_device);
+		printf("#\n# Logical device %d\n", logical_device);
 		for (addr = 0x40, desc = 0; addr < 0x60; addr += 8, desc++)
 		{
-			printf("24 bit Memory descriptor %d at %02x..%02x: ", desc, addr, addr + 4);
+			printf("# 24 bit Memory descriptor %d at %02x..%02x: ", desc, addr, addr + 4);
 			for (i = 0; i < 5; i++)
 			{
 				ADDRESS(addr + i);
@@ -1910,7 +1967,7 @@ dumpregs(int csn)
 		}
 		for (addr = 0x76, desc = 0; addr < 0xb0; addr += 16, desc++)
 		{
-			printf("32 bit Memory descriptor %d at %02x..%02x: ", desc, addr, addr + 8);
+			printf("# 32 bit Memory descriptor %d at %02x..%02x: ", desc, addr, addr + 8);
 			for (i = 0; i < 9; i++)
 			{
 				ADDRESS(addr + i);
@@ -1923,7 +1980,7 @@ dumpregs(int csn)
 		}
 		for (addr = 0x60, desc = 0; addr < 0x70; addr += 2, desc++)
 		{
-			printf("IO descriptor %d at %02x..%02x: ", desc, addr, addr + 1);
+			printf("# IO descriptor %d at %02x..%02x: ", desc, addr, addr + 1);
 			for (i = 0; i < 2; i++)
 			{
 				ADDRESS(addr + i);
@@ -1933,7 +1990,7 @@ dumpregs(int csn)
 		}
 		for (addr = 0x70, desc = 0; addr < 0x74; addr += 2, desc++)
 		{
-			printf("Interrupt descriptor %d at %02x..%02x: ", desc, addr, addr + 1);
+			printf("# Interrupt descriptor %d at %02x..%02x: ", desc, addr, addr + 1);
 			for (i = 0; i < 2; i++)
 			{
 				ADDRESS(addr + i);
@@ -1943,12 +2000,11 @@ dumpregs(int csn)
 		}
 		for (addr = 0x74; addr < 0x76; addr++)
 		{
-			printf("DMA descriptor %d at %02x: ", addr - 0x74, addr);
+			printf("# DMA descriptor %d at %02x: ", addr - 0x74, addr);
 			ADDRESS(addr);
 			tmp[0] = READ_DATA;
 			printf("DMA channel %d\n", tmp[0]);
 		}
 	}
+	putchar('\n');
 }
-
-#endif /* DUMPREGS */
